/*
 * TableSample.java
 *
 * Created on 2010.12.01 - 12:44:33
 *
 */
package org.openoffice.sdk.text.table;

import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.uno.Exception;
import com.sun.star.uno.XComponentContext;
import com.sun.star.comp.helper.Bootstrap;
import com.sun.star.frame.FrameSearchFlag;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XController;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.table.XCell;
import com.sun.star.table.XCellRange;
import com.sun.star.text.ControlCharacter;
import com.sun.star.text.XText;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextTable;
import com.sun.star.text.XTextTableCursor;
import com.sun.star.uno.UnoRuntime;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author ariel
 */
public class TableSample {

    private final XComponentContext m_xContext;
    private final int m_nRowCount;
    private final Boolean m_bHidden;
    private boolean m_bDone = false;
    private long m_nRunTime, m_nTableTime, m_nCellTime;

    /** Creates a new instance of TableSample */
    public TableSample(XComponentContext xContext, int nRows, boolean bHidden) {
        m_xContext = xContext;
        m_nRowCount = nRows;
        m_bHidden = Boolean.valueOf(bHidden);
    }

    public void printStats() {
        if (!m_bDone)
            return;
        System.out.printf("Total Time:               %d\n", m_nRunTime);
        System.out.printf("Time inserting the table: %d\n", m_nTableTime);
        System.out.printf("Time filling the cells:   %d\n", m_nCellTime);
    }

    public void runDemo() {

        long nRunStart = System.nanoTime();

        XTextDocument xTextDocument = createDocument();
        XMultiServiceFactory xDocFactory = (XMultiServiceFactory) UnoRuntime.queryInterface(
                XMultiServiceFactory.class, xTextDocument);
        if ( xDocFactory == null )
            return;

        if (!m_bHidden)
            xTextDocument.lockControllers();
        try {
            XTextTable xTextTable = (XTextTable) UnoRuntime.queryInterface(
                    XTextTable.class,
                    xDocFactory.createInstance("com.sun.star.text.TextTable"));
            xTextTable.initialize(m_nRowCount, 2);

            XCellRange xCellRange = (XCellRange) UnoRuntime.queryInterface(XCellRange.class, xTextTable);

            XTextCursor xTextCursor = xTextDocument.getText().createTextCursorByRange(xTextDocument.getText().getStart());
            XPropertySet xCursorProps = (XPropertySet) UnoRuntime.queryInterface(XPropertySet.class, xTextCursor);

            xCursorProps.setPropertyValue("ParaStyleName", "Heading 1");
            xTextCursor.getText().insertString(xTextCursor, "Text Table Sample", false);
            xTextCursor.getText().insertControlCharacter(xTextCursor, ControlCharacter.PARAGRAPH_BREAK, false);
            xCursorProps.setPropertyValue("ParaStyleName", "Default");

            long nTableTime = System.nanoTime();
            xTextCursor.getText().insertTextContent(xTextCursor, xTextTable, false);
            m_nTableTime = System.nanoTime() - nTableTime;
            
            long nCellTime = System.nanoTime();
            XTextTableCursor xTableCursor = xTextTable.createCursorByCellName("A1");

            for (int n = 0; n < m_nRowCount; n++) {
                XCell xCell = xCellRange.getCellByPosition(0, n);
                XText xCellText = (XText) UnoRuntime.queryInterface(
                        XText.class, xCell);
                xCellText.setString(xTableCursor.getRangeName());

                xCell = xCellRange.getCellByPosition(1, n);
                xCell.setValue((double) (n+1));

                xTableCursor.goDown((short) 1, false);
            }

            m_nCellTime = System.nanoTime() - nCellTime;

        } catch (Exception ex) {
            Logger.getLogger(TableSample.class.getName()).log(Level.SEVERE, null, ex);
        }

        if (!m_bHidden)
            xTextDocument.unlockControllers();
        else
        {
            XController xController = xTextDocument.getCurrentController();
            xController.getFrame().getContainerWindow().setVisible(true);
        }
        m_nRunTime = System.nanoTime() - nRunStart;
        m_bDone = true;
    }

    private XTextDocument createDocument() {
        XTextDocument xRet = null;
        try {
            XComponentLoader xLoader = (XComponentLoader) UnoRuntime.queryInterface(
                    XComponentLoader.class,
                    m_xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.frame.Desktop", m_xContext));

            PropertyValue[] aMediaDescriptor = new PropertyValue[1];
            aMediaDescriptor[0] = new PropertyValue();
            aMediaDescriptor[0].Name = "Hidden";
            aMediaDescriptor[0].Value = m_bHidden;

            xRet = (XTextDocument) UnoRuntime.queryInterface(
                    XTextDocument.class,
                    xLoader.loadComponentFromURL(
                    "private:factory/swriter",
                    "_default",
                    FrameSearchFlag.ALL,
                    aMediaDescriptor));
        } catch (Exception e) {
        } finally {
            return xRet;
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            // get the remote office component context
            XComponentContext xContext = Bootstrap.bootstrap();
            if (xContext != null) {
                int nRowCount = 3000;
                TableSample sample = new TableSample(xContext, nRowCount, true);
                sample.runDemo();
                sample.printStats();
            }
        } catch (java.lang.Exception e) {
            e.printStackTrace();
        } finally {
            System.exit(0);
        }
    }
}
